"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const file_1 = require("../../libs/file");
const format_1 = require("../../libs/format");
const logger_1 = require("../../libs/logger");
const progress_bar_1 = require("../../libs/progress-bar");
const variables_manager_1 = require("../../managers/variables-manager");
const web_1 = require("stream/web");
const head_1 = require("./head");
class Get extends head_1.Head {
    parent;
    method = 'get';
    responseType = 'json';
    saveTo;
    async send() {
        if (this.saveTo) {
            this.responseType = 'pipe';
            this.saveTo = variables_manager_1.GlobalUtils.getPath(this.saveTo);
        }
        this.prehandleHeaders();
        const rs = await fetch(this.fullURLQuery, {
            method: this.method,
            headers: this.headers,
        });
        const data = await this.getResponseData(rs);
        this.response = {
            ok: rs.ok,
            headers: this.getResponseHeader(rs),
            data,
            status: rs.status,
            statusText: rs.statusText,
        };
        return this.response;
    }
    async getResponseData(rs) {
        switch (this.responseType) {
            case 'none':
                return undefined;
            case 'json':
                return rs.json();
            case 'blob':
                return rs.blob();
            case 'buffer':
                return rs.arrayBuffer();
            case 'pipe':
                return new Promise(async (resolve, reject) => {
                    const bar = this.logger.is(logger_1.LoggerLevel.INFO) ? new progress_bar_1.ProgressBar('Downloading...', this.logger.indent) : undefined;
                    bar?.start();
                    const stream = (0, fs_1.createWriteStream)(this.saveTo, { autoClose: false, emitClose: false });
                    try {
                        await rs.body.pipeTo(new web_1.WritableStream({
                            total: 0,
                            write(chunk) {
                                if (bar) {
                                    this.total += chunk.byteLength;
                                    bar.update(`Downloaded ${(0, format_1.formatNumber)(this.total / 1000, { maximumFractionDigits: 0 })} kbs | Rate: ${(0, format_1.formatNumber)(chunk.byteLength, { maximumFractionDigits: 0 })} bytes`);
                                }
                                return stream.write(chunk);
                            }
                        }));
                        stream.on('error', err => {
                            reject(err);
                        });
                        stream.end(() => {
                            bar?.stop();
                            resolve(new file_1.File(this.saveTo));
                        });
                    }
                    catch (err) {
                        bar?.failed(this.saveTo);
                        reject(err);
                    }
                    finally {
                        bar?.destroy();
                    }
                });
            default:
                return rs.text();
        }
    }
}
exports.default = Get;
//# sourceMappingURL=get.js.map