"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Head = void 0;
const chalk_1 = __importDefault(require("chalk"));
const querystring_1 = require("querystring");
const http_error_1 = require("./http-error");
class Head {
    method = 'head';
    title;
    baseURL;
    url;
    headers = {};
    query;
    response;
    vars;
    executionTime;
    error;
    logger;
    get fullURL() {
        return `${this.baseURL || ''}${this.url}`;
    }
    get fullURLQuery() {
        return this.fullURL + (this.query ? `?${(0, querystring_1.encode)(this.query)}` : '');
    }
    async init(props) {
        Object.assign(this, props);
    }
    async dispose() { }
    async exec() {
        try {
            this.logger.debugBlock(true);
            this.logger.debug(`%s \t%s`, chalk_1.default.gray('‣ Method'), this.method.toUpperCase());
            this.logger.debug(`%s    \t%s`, chalk_1.default.gray('‣ URL'), this.fullURL);
            this.prehandleQuery();
            const before = Date.now();
            await this.send();
            this.executionTime = Date.now() - before;
            this.prehandleResponseHeaders();
            this.prehandleResponseData();
            if (!this.response.ok) {
                this.error = new http_error_1.HttpError(this.response.status, this.response.statusText, this.response.data);
            }
        }
        catch (err) {
            this.error = new http_error_1.HttpError(0, err.message);
        }
        finally {
            this.vars && this.applyVar();
            this.logger.debugBlock(false);
        }
        if (this.error) {
            if (this.error.status >= 500 || !this.error.status)
                throw this.error;
            this.logger.error(this.error);
        }
        return this.response?.data;
    }
    async send() {
        this.prehandleHeaders();
        const rs = await fetch(this.fullURLQuery, {
            method: this.method,
            headers: this.headers,
        });
        this.response = {
            ok: rs.ok,
            headers: this.getResponseHeader(rs),
            status: rs.status,
            statusText: rs.statusText,
        };
        return this.response;
    }
    applyVar() {
        let varNames;
        if (typeof this.vars === 'string') {
            varNames = this.vars;
        }
        else {
            varNames = Object.keys(this.vars).map(v => `${v}`).join(', ');
        }
        this.logger.debug('');
        this.logger.debug(`%s   \t%s`, chalk_1.default.gray('‣ Vars'), varNames);
    }
    checkEmpty(vl) {
        if (!vl)
            return undefined;
        if (Array.isArray(vl) && !vl.length)
            return undefined;
        if (typeof vl === 'object' && !Object.keys(vl).length)
            return undefined;
        return vl;
    }
    setHeader(key, value) {
        if (!this.headers)
            this.headers = {};
        if (!this.headers[key])
            this.headers[key] = value;
    }
    getResponseHeader(rs) {
        const headers = Array.from(rs.headers.entries());
        if (!headers.length)
            return undefined;
        return headers.reduce((sum, [key, vl]) => {
            sum[key] = vl;
            return sum;
        }, {});
    }
    prehandleResponseHeaders() {
        const color = this.response?.status ? chalk_1.default.green : chalk_1.default.red;
        const icon = this.response?.status ? '✔' : '✘';
        this.logger.debug('');
        this.logger.debug(`${color(`%s %d %s`)} \t%s`, icon, this.response.status, this.response.statusText, chalk_1.default.gray(`${this.executionTime}ms`));
        this.response.headers = this.checkEmpty(this.response.headers);
        this.response?.headers && this.logger.debug(`%s\t%j`, chalk_1.default.gray('‣ Headers'), this.response?.headers);
    }
    prehandleResponseData() {
        this.response?.data && this.logger.debug(`%s   \t%j`, chalk_1.default.gray('‣ Data'), this.response.data);
    }
    prehandleHeaders() {
        this.headers = this.checkEmpty(this.headers);
        this.headers && this.logger.debug(`%s\t%j`, chalk_1.default.gray('‣ Headers'), this.headers);
    }
    prehandleQuery() {
        const [url, queryString] = this.url.split('?');
        this.url = url;
        if (queryString) {
            if (!this.query)
                this.query = {};
            const query = (0, querystring_1.decode)(queryString);
            Object.assign(this.query, query);
        }
        this.query = this.checkEmpty(this.query);
        this.query && this.logger.debug(`%s  \t%j`, chalk_1.default.gray('‣ Query'), this.query);
    }
}
exports.Head = Head;
//# sourceMappingURL=head.js.map